/*
  This code is part of FCPTools - an FCP-based client library for Freenet

  CopyLeft (c) 2001 by David McNab

  Developers:
  - David McNab <david@rebirthing.co.nz>
  - Jay Oliveri <ilnero@gmx.net>
  
  Currently maintained by Jay Oliveri <ilnero@gmx.net>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "ezFCPlib.h"

#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include "ez_sys.h"

/* define all the tests you wish to run and all of them will run one after
	 another */

#define LOG_LEVEL 4

#define SINGLE_FILE_ONE
#define SINGLE_FILE_TWO

/*#define SPLIT_FILE*/

#if 0
int main(int argc, char *argv[])
{
  hFCP *hfcp;
  int   rc;
  
  char  buf[8193];
  int   bytes;

	char *uri_str;
  
  rc = 0;
  
  if (fcpStartup(stdout, LOG_LEVEL)) {
    fprintf(stdout, "Failed to initialize FCPLib\n");
    rc = -1;
		goto cleanup;
  }
	
  hfcp = fcpCreateHFCP("127.0.0.1", 8481, 0, 0);

 	/********************************************************************/
	/* */
	/********************************************************************/

#ifdef SINGLE_FILE_ONE
	printf("\nSINGLE_FILE_ONE test: (fcpOpenKey(), ReadKey(), WriteKey()\n");
	printf("CHK\n\n");

	if (fcpOpenKey(hfcp, "CHK@", FCPT_MODE_O_WRITE)) {
		rc = -1;
		goto cleanup;
	}
	
	strcpy(buf, "This is a test");
	fcpWriteKey(hfcp, buf, strlen(buf));
	
	if (fcpCloseKey(hfcp)) {
		rc = -1;
		goto cleanup;
	}
	
	uri_str = strdup(hfcp->key->target_uri->uri_str);
	fprintf(stdout, "%s\n", uri_str);

	if (fcpOpenKey(hfcp, uri_str, FCPT_MODE_O_READ)) {
		rc = -1;
		goto cleanup;
	}

	buf[fcpReadKey(hfcp, buf, 8192)] = 0;
	printf("DATA...\n%s\n", buf);
	
	buf[fcpReadMetadata(hfcp, buf, 8192)] = 0;
	printf("METADATA...\n%s\n", buf);

	if (fcpCloseKey(hfcp)) {
		rc = -1;
		goto cleanup;
	}

	printf("KSK\n\n");

	if (fcpOpenKey(hfcp, "KSK@testing-it-again", FCPT_MODE_O_WRITE)) {
		rc = -1;
		goto cleanup;
	}
	
	strcpy(buf, "This is a KSK test");
	fcpWriteKey(hfcp, buf, strlen(buf));
	
	if (fcpCloseKey(hfcp)) {
		rc = -1;
		goto cleanup;
	}
	
	uri_str = strdup(hfcp->key->target_uri->uri_str);
	fprintf(stdout, "%s\n", uri_str);

	if (fcpOpenKey(hfcp, uri_str, FCPT_MODE_O_READ)) {
		rc = -1;
		goto cleanup;
	}

	buf[fcpReadKey(hfcp, buf, 8192)] = 0;
	printf("DATA...\n%s\n", buf);
	
	buf[fcpReadMetadata(hfcp, buf, 8192)] = 0;
	printf("METADATA...\n%s\n", buf);

	if (fcpCloseKey(hfcp)) {
		rc = -1;
		goto cleanup;
	}

#endif

 	/********************************************************************/
	/* CLEANUP; then exit the test program                              */
	/********************************************************************/
	
	/* make sure we enter 'cleanup' with a success value; all others with errors (!0) */
	rc = 0;

 cleanup:
	
	fcpDestroyHFCP(hfcp);
	free(hfcp);

  fcpTerminate();

	if (uri_str) free(uri_str);
	return rc;
}
#endif
